@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 """Parses Doxygen XML output to generate Neovim's API documentation.
 
 This would be easier using lxml and XSLT, but:
@@ -45,6 +45,8 @@ if sys.version_info[0] < 3:
 doc_filename = 'api.txt'
 # String used to find the start of the generated part of the doc.
 section_start_token = '*api-global*'
+# Required prefix for API function names.
+api_func_name_prefix = 'nvim_'
 
 # Section name overrides.
 section_name = {
@@ -186,7 +188,7 @@ def parse_params(parent, width=62):
             desc = parse_parblock(desc_node, width=None)
         items.append((name.strip(), desc.strip()))
 
-    out = 'Parameters:~\n'
+    out = 'Parameters: ~\n'
     for name, desc in items:
         name = '    %s' % name.ljust(name_length)
         out += doc_wrap(desc, prefix=name, width=width) + '\n'
@@ -227,7 +229,7 @@ def parse_para(parent, width=62):
                                           prefix='    ',
                                           width=width))
                 elif kind == 'return':
-                    lines.append('%s:~' % kind.title())
+                    lines.append('%s: ~' % kind.title())
                     lines.append(doc_wrap(parse_para(child),
                                           prefix='    ',
                                           width=width))
@@ -260,11 +262,11 @@ def parse_parblock(parent, width=62):
 def parse_source_xml(filename):
     """Collects API functions.
 
-    This returns two strings:
-      1. The API functions
-      2. The deprecated API functions
+    Returns two strings:
+      1. API functions
+      2. Deprecated API functions
 
-    The caller decides what to do with the deprecated documentation.
+    Caller decides what to do with the deprecated documentation.
     """
     global xrefs
     xrefs = set()
@@ -294,9 +296,8 @@ def parse_source_xml(filename):
         annotations = get_text(get_child(member, 'argsstring'))
         if annotations and ')' in annotations:
             annotations = annotations.rsplit(')', 1)[-1].strip()
-        # XXX: (doxygen 1.8.11) 'argsstring' only includes FUNC_ATTR_*
-        # attributes if the function signature is non-void.
-        # Force attributes here for such functions.
+        # XXX: (doxygen 1.8.11) 'argsstring' only includes attributes of
+        # non-void functions.  Special-case void functions here.
         if name == 'nvim_get_mode' and len(annotations) == 0:
             annotations += 'FUNC_API_ASYNC'
         annotations = filter(None, map(lambda x: annotation_map.get(x),
@@ -360,16 +361,16 @@ def parse_source_xml(filename):
 
         annotations = '\n'.join(annotations)
         if annotations:
-            annotations = ('\n\nAttributes:~\n' +
+            annotations = ('\n\nAttributes: ~\n' +
                            textwrap.indent(annotations, '    '))
-            i = doc.rfind('Parameters:~')
+            i = doc.rfind('Parameters: ~')
             if i == -1:
                 doc += annotations
             else:
                 doc = doc[:i] + annotations + '\n\n' + doc[i:]
 
         if 'INCLUDE_C_DECL' in os.environ:
-            doc += '\n\nC Declaration:~\n>\n'
+            doc += '\n\nC Declaration: ~\n>\n'
             doc += c_decl
             doc += '\n<'
 
@@ -379,7 +380,7 @@ def parse_source_xml(filename):
 
         if 'Deprecated' in xrefs:
             deprecated_functions.append(func_doc)
-        else:
+        elif name.startswith(api_func_name_prefix):
             functions.append(func_doc)
 
         xrefs.clear()
@@ -440,7 +441,7 @@ def gen_docs(config):
                     doc += '\n\n' + functions
 
                 if 'INCLUDE_DEPRECATED' in os.environ and deprecated:
-                    doc += '\n\n\nDeprecated %s Functions:~\n\n' % name
+                    doc += '\n\n\nDeprecated %s Functions: ~\n\n' % name
                     doc += deprecated
 
                 if doc:
@@ -477,7 +478,7 @@ def gen_docs(config):
             docs += '\n\n\n'
 
     docs = docs.rstrip() + '\n\n'
-    docs += ' vim:tw=78:ts=8:ft=help:norl:'
+    docs += ' vim:tw=78:ts=8:ft=help:norl:\n'
 
     doc_file = os.path.join(base_dir, 'runtime/doc', doc_filename)
     delete_lines_below(doc_file, section_start_token)
