@@ -29,17 +29,20 @@ static void msgpack_rpc_add_method_handler(String method,
   map_put(String, MsgpackRpcRequestHandler)(methods, method, handler);
 }
 
+/// @param name API method name
+/// @param name_len name size (includes terminating NUL)
 MsgpackRpcRequestHandler msgpack_rpc_get_handler_for(const char *name,
-                                                     size_t name_len)
+                                                     size_t name_len,
+                                                     Error *error)
 {
   String m = { .data = (char *)name, .size = name_len };
   MsgpackRpcRequestHandler rv =
     map_get(String, MsgpackRpcRequestHandler)(methods, m);
 
   if (!rv.fn) {
-    rv.fn = msgpack_rpc_handle_missing_method;
+    api_set_error(error, kErrorTypeException, "Invalid method: %s",
+                  m.size > 0 ? m.data : "<empty>");
   }
-
   return rv;
 }
 
