@@ -1541,26 +1541,30 @@ void do_autocmd_dirchanged(char *new_dir, CdScope scope)
   char buf[8];
 
   switch (scope) {
-  case kCdScopeGlobal:
-    snprintf(buf, sizeof(buf), "global");
-    break;
-  case kCdScopeTab:
-    snprintf(buf, sizeof(buf), "tab");
-    break;
-  case kCdScopeWindow:
-    snprintf(buf, sizeof(buf), "window");
-    break;
-  case kCdScopeInvalid:
-    // Should never happen.
-    assert(false);
+    case kCdScopeGlobal: {
+      snprintf(buf, sizeof(buf), "global");
+      break;
+    }
+    case kCdScopeTab: {
+      snprintf(buf, sizeof(buf), "tab");
+      break;
+    }
+    case kCdScopeWindow: {
+      snprintf(buf, sizeof(buf), "window");
+      break;
+    }
+    case kCdScopeInvalid: {
+      // Should never happen.
+      assert(false);
+    }
   }
 
-  tv_dict_add_str(dict, S_LEN("scope"), buf);
+  tv_dict_add_str(dict, S_LEN("scope"), buf);  // -V614
   tv_dict_add_str(dict, S_LEN("cwd"),   new_dir);
   tv_dict_set_keys_readonly(dict);
 
   apply_autocmds(EVENT_DIRCHANGED, (char_u *)buf, (char_u *)new_dir, false,
-                 NULL);
+                 curbuf);
 
   tv_dict_clear(dict);
 
@@ -1586,7 +1590,7 @@ int vim_chdirfile(char_u *fname)
   }
 
 #ifdef BACKSLASH_IN_FILENAME
-  slash_adjust(dir);
+  slash_adjust((char_u *)dir);
 #endif
   if (!strequal(dir, (char *)NameBuff)) {
     do_autocmd_dirchanged(dir, kCdScopeWindow);
