@@ -584,7 +584,7 @@ static void prt_header(prt_settings_T *psettings, int pagenum, linenr_T lnum)
 static void prt_message(char_u *s)
 {
   screen_fill((int)Rows - 1, (int)Rows, 0, (int)Columns, ' ', ' ', 0);
-  screen_puts(s, (int)Rows - 1, 0, hl_attr(HLF_R));
+  screen_puts(s, (int)Rows - 1, 0, HL_ATTR(HLF_R));
   ui_flush();
 }
 
