@@ -1,7 +1,7 @@
 local helpers = require('test.functional.helpers')(after_each)
-local eq, clear, eval, command, nvim, next_message =
+local eq, clear, eval, command, nvim, next_msg =
   helpers.eq, helpers.clear, helpers.eval, helpers.command, helpers.nvim,
-  helpers.next_message
+  helpers.next_msg
 local meths = helpers.meths
 
 describe('notify', function()
@@ -15,10 +15,10 @@ describe('notify', function()
   describe('passing a valid channel id', function()
     it('sends the notification/args to the corresponding channel', function()
       eval('rpcnotify('..channel..', "test-event", 1, 2, 3)')
-      eq({'notification', 'test-event', {1, 2, 3}}, next_message())
+      eq({'notification', 'test-event', {1, 2, 3}}, next_msg())
       command('au FileType lua call rpcnotify('..channel..', "lua!")')
       command('set filetype=lua')
-      eq({'notification', 'lua!', {}}, next_message())
+      eq({'notification', 'lua!', {}}, next_msg())
     end)
   end)
 
@@ -28,13 +28,13 @@ describe('notify', function()
       eval('rpcnotify(0, "event1", 1, 2, 3)')
       eval('rpcnotify(0, "event2", 4, 5, 6)')
       eval('rpcnotify(0, "event2", 7, 8, 9)')
-      eq({'notification', 'event2', {4, 5, 6}}, next_message())
-      eq({'notification', 'event2', {7, 8, 9}}, next_message())
+      eq({'notification', 'event2', {4, 5, 6}}, next_msg())
+      eq({'notification', 'event2', {7, 8, 9}}, next_msg())
       nvim('unsubscribe', 'event2')
       nvim('subscribe', 'event1')
       eval('rpcnotify(0, "event2", 10, 11, 12)')
       eval('rpcnotify(0, "event1", 13, 14, 15)')
-      eq({'notification', 'event1', {13, 14, 15}}, next_message())
+      eq({'notification', 'event1', {13, 14, 15}}, next_msg())
     end)
 
     it('does not crash for deeply nested variable', function()
@@ -42,7 +42,7 @@ describe('notify', function()
       local nest_level = 1000
       meths.command(('call map(range(%u), "extend(g:, {\'l\': [g:l]})")'):format(nest_level - 1))
       eval('rpcnotify('..channel..', "event", g:l)')
-      local msg = next_message()
+      local msg = next_msg()
       eq('notification', msg[1])
       eq('event', msg[2])
       local act_ret = msg[3]
