@@ -6,11 +6,12 @@ local Paths = require('test.config.paths')
 local clear, nvim, eval = helpers.clear, helpers.nvim, helpers.eval
 local eq, neq, run, stop = helpers.eq, helpers.neq, helpers.run, helpers.stop
 local nvim_prog, command, funcs = helpers.nvim_prog, helpers.command, helpers.funcs
-local source, next_message = helpers.source, helpers.next_message
+local source, next_msg = helpers.source, helpers.next_msg
 local ok = helpers.ok
 local meths = helpers.meths
 local spawn, nvim_argv = helpers.spawn, helpers.nvim_argv
 local set_session = helpers.set_session
+local expect_err = helpers.expect_err
 
 describe('server -> client', function()
   local cid
@@ -221,9 +222,8 @@ describe('server -> client', function()
     end)
 
     it('returns an error if the request failed', function()
-      local status, err = pcall(eval, "rpcrequest(vim, 'does-not-exist')")
-      eq(false, status)
-      ok(nil ~= string.match(err, 'Failed to evaluate expression'))
+      expect_err('Vim:Invalid method: does%-not%-exist',
+                 eval, "rpcrequest(vim, 'does-not-exist')")
     end)
   end)
 
@@ -250,7 +250,7 @@ describe('server -> client', function()
     end)
 
     after_each(function()
-      funcs.jobstop(jobid)
+      pcall(funcs.jobstop, jobid)
     end)
 
     if helpers.pending_win32(pending) then return end
@@ -258,11 +258,12 @@ describe('server -> client', function()
     it('rpc and text stderr can be combined', function()
       eq("ok",funcs.rpcrequest(jobid, "poll"))
       funcs.rpcnotify(jobid, "ping")
-      eq({'notification', 'pong', {}}, next_message())
+      eq({'notification', 'pong', {}}, next_msg())
       eq("done!",funcs.rpcrequest(jobid, "write_stderr", "fluff\n"))
-      eq({'notification', 'stderr', {0, {'fluff', ''}}}, next_message())
-      funcs.rpcrequest(jobid, "exit")
-      eq({'notification', 'exit', {0, 0}}, next_message())
+      eq({'notification', 'stderr', {0, {'fluff', ''}}}, next_msg())
+      pcall(funcs.rpcrequest, jobid, "exit")
+      eq({'notification', 'stderr', {0, {''}}}, next_msg())
+      eq({'notification', 'exit', {0, 0}}, next_msg())
     end)
   end)
 
@@ -307,8 +308,8 @@ describe('server -> client', function()
     it('via ipv4 address', function()
       local server = spawn(nvim_argv)
       set_session(server)
-      local address = funcs.serverstart("127.0.0.1:")
-      if #address == 0 then
+      local status, address = pcall(funcs.serverstart, "127.0.0.1:")
+      if not status then
         pending('no ipv4 stack', function() end)
         return
       end
@@ -319,8 +320,8 @@ describe('server -> client', function()
     it('via ipv6 address', function()
       local server = spawn(nvim_argv)
       set_session(server)
-      local address = funcs.serverstart('::1:')
-      if #address == 0 then
+      local status, address = pcall(funcs.serverstart, '::1:')
+      if not status then
         pending('no ipv6 stack', function() end)
         return
       end
