@@ -2,24 +2,35 @@ local helpers = require('test.functional.helpers')(after_each)
 local clear, eq, eval, exc_exec, feed_command, feed, insert, neq, next_msg, nvim,
   nvim_dir, ok, source, write_file, mkdir, rmdir = helpers.clear,
   helpers.eq, helpers.eval, helpers.exc_exec, helpers.feed_command, helpers.feed,
-  helpers.insert, helpers.neq, helpers.next_message, helpers.nvim,
+  helpers.insert, helpers.neq, helpers.next_msg, helpers.nvim,
   helpers.nvim_dir, helpers.ok, helpers.source,
   helpers.write_file, helpers.mkdir, helpers.rmdir
 local command = helpers.command
+local funcs = helpers.funcs
+local retry = helpers.retry
+local meths = helpers.meths
+local NIL = helpers.NIL
 local wait = helpers.wait
 local iswin = helpers.iswin
 local get_pathsep = helpers.get_pathsep
+local pathroot = helpers.pathroot
 local nvim_set = helpers.nvim_set
+local expect_twostreams = helpers.expect_twostreams
+local expect_msg_seq = helpers.expect_msg_seq
 local Screen = require('test.functional.ui.screen')
 
+-- Kill process with given pid
+local function os_kill(pid)
+  return os.execute((iswin()
+    and 'taskkill /f /t /pid '..pid..' > nul'
+    or  'kill -9 '..pid..' > /dev/null'))
+end
+
 describe('jobs', function()
   local channel
 
   before_each(function()
     clear()
-    if iswin() then
-      helpers.set_shell_powershell()
-    end
     channel = nvim('get_api_info')[1]
     nvim('set_var', 'channel', channel)
     source([[
@@ -29,15 +40,14 @@ describe('jobs', function()
         \ ? map(a:data, 'substitute(v:val, "\r", "", "g")')
         \ : a:data
     endfunction
-    function! s:OnEvent(id, data, event) dict
+    function! OnEvent(id, data, event) dict
       let userdata = get(self, 'user')
       let data     = Normalize(a:data)
       call rpcnotify(g:channel, a:event, userdata, data)
     endfunction
     let g:job_opts = {
-    \ 'on_stdout': function('s:OnEvent'),
-    \ 'on_stderr': function('s:OnEvent'),
-    \ 'on_exit': function('s:OnEvent'),
+    \ 'on_stdout': function('OnEvent'),
+    \ 'on_exit': function('OnEvent'),
     \ 'user': 0
     \ }
     ]])
@@ -46,23 +56,25 @@ describe('jobs', function()
   it('uses &shell and &shellcmdflag if passed a string', function()
     nvim('command', "let $VAR = 'abc'")
     if iswin() then
-      nvim('command', "let j = jobstart('echo $env:VAR', g:job_opts)")
+      nvim('command', "let j = jobstart('echo %VAR%', g:job_opts)")
     else
       nvim('command', "let j = jobstart('echo $VAR', g:job_opts)")
     end
     eq({'notification', 'stdout', {0, {'abc', ''}}}, next_msg())
+    eq({'notification', 'stdout', {0, {''}}}, next_msg())
     eq({'notification', 'exit', {0, 0}}, next_msg())
   end)
 
   it('changes to given / directory', function()
     nvim('command', "let g:job_opts.cwd = '/'")
     if iswin() then
-      nvim('command', "let j = jobstart('pwd|%{$_.Path}', g:job_opts)")
+      nvim('command', "let j = jobstart('cd', g:job_opts)")
     else
       nvim('command', "let j = jobstart('pwd', g:job_opts)")
     end
     eq({'notification', 'stdout',
-      {0, {(iswin() and [[C:\]] or '/'), ''}}}, next_msg())
+      {0, {pathroot(), ''}}}, next_msg())
+    eq({'notification', 'stdout', {0, {''}}}, next_msg())
     eq({'notification', 'exit', {0, 0}}, next_msg())
   end)
 
@@ -71,12 +83,22 @@ describe('jobs', function()
     mkdir(dir)
     nvim('command', "let g:job_opts.cwd = '" .. dir .. "'")
     if iswin() then
-      nvim('command', "let j = jobstart('pwd|%{$_.Path}', g:job_opts)")
+      nvim('command', "let j = jobstart('cd', g:job_opts)")
     else
       nvim('command', "let j = jobstart('pwd', g:job_opts)")
     end
-    eq({'notification', 'stdout', {0, {dir, ''}}}, next_msg())
-    eq({'notification', 'exit', {0, 0}}, next_msg())
+    expect_msg_seq(
+      { {'notification', 'stdout', {0, {dir, ''} } },
+        {'notification', 'stdout', {0, {''} } },
+        {'notification', 'exit', {0, 0} }
+      },
+      -- Alternative sequence:
+      { {'notification', 'stdout', {0, {dir} } },
+        {'notification', 'stdout', {0, {'', ''} } },
+        {'notification', 'stdout', {0, {''} } },
+        {'notification', 'exit', {0, 0} }
+      }
+    )
     rmdir(dir)
   end)
 
@@ -85,7 +107,7 @@ describe('jobs', function()
     local _, err = pcall(function()
       nvim('command', "let g:job_opts.cwd = '" .. dir .. "'")
       if iswin() then
-        nvim('command', "let j = jobstart('pwd|%{$_.Path}', g:job_opts)")
+        nvim('command', "let j = jobstart('cd', g:job_opts)")
       else
         nvim('command', "let j = jobstart('pwd', g:job_opts)")
       end
@@ -101,13 +123,13 @@ describe('jobs', function()
   end)
 
   it('returns -1 when target is not executable #5465', function()
-    if helpers.pending_win32(pending) then return end
     local function new_job()
       return eval([[jobstart('')]])
     end
     local executable_jobid = new_job()
-    local nonexecutable_jobid = eval(
-      "jobstart(['./test/functional/fixtures/non_executable.txt'])")
+    local nonexecutable_jobid = eval("jobstart(['"..(iswin()
+      and './test/functional/fixtures'
+      or  './test/functional/fixtures/non_executable.txt').."'])")
     eq(-1, nonexecutable_jobid)
     -- Should _not_ throw an error.
     eq("", eval("v:errmsg"))
@@ -116,24 +138,39 @@ describe('jobs', function()
   end)
 
   it('invokes callbacks when the job writes and exits', function()
-    -- TODO: hangs on Windows
-    if helpers.pending_win32(pending) then return end
-    nvim('command', "call jobstart('echo', g:job_opts)")
-    eq({'notification', 'stdout', {0, {'', ''}}}, next_msg())
+    nvim('command', "let g:job_opts.on_stderr  = function('OnEvent')")
+    nvim('command', [[call jobstart(has('win32') ? 'echo:' : 'echo', g:job_opts)]])
+    expect_twostreams({{'notification', 'stdout', {0, {'', ''}}},
+                       {'notification', 'stdout', {0, {''}}}},
+                      {{'notification', 'stderr', {0, {''}}}})
     eq({'notification', 'exit', {0, 0}}, next_msg())
   end)
 
   it('allows interactive commands', function()
-    if helpers.pending_win32(pending) then return end  -- TODO: Need `cat`.
-    nvim('command', "let j = jobstart(['cat', '-'], g:job_opts)")
+    nvim('command', "let j = jobstart(has('win32') ? ['find', '/v', ''] : ['cat', '-'], g:job_opts)")
     neq(0, eval('j'))
     nvim('command', 'call jobsend(j, "abc\\n")')
     eq({'notification', 'stdout', {0, {'abc', ''}}}, next_msg())
     nvim('command', 'call jobsend(j, "123\\nxyz\\n")')
-    eq({'notification', 'stdout', {0, {'123', 'xyz', ''}}}, next_msg())
+    expect_msg_seq(
+      { {'notification', 'stdout', {0, {'123', 'xyz', ''}}}
+      },
+      -- Alternative sequence:
+      { {'notification', 'stdout', {0, {'123', ''}}},
+        {'notification', 'stdout', {0, {'xyz', ''}}}
+      }
+    )
     nvim('command', 'call jobsend(j, [123, "xyz", ""])')
-    eq({'notification', 'stdout', {0, {'123', 'xyz', ''}}}, next_msg())
+    expect_msg_seq(
+      { {'notification', 'stdout', {0, {'123', 'xyz', ''}}}
+      },
+      -- Alternative sequence:
+      { {'notification', 'stdout', {0, {'123', ''}}},
+        {'notification', 'stdout', {0, {'xyz', ''}}}
+      }
+    )
     nvim('command', "call jobstop(j)")
+    eq({'notification', 'stdout', {0, {''}}}, next_msg())
     eq({'notification', 'exit', {0, 0}}, next_msg())
   end)
 
@@ -145,6 +182,7 @@ describe('jobs', function()
 
     nvim('command', "let j = jobstart(['cat', '"..filename.."'], g:job_opts)")
     eq({'notification', 'stdout', {0, {'abc\ndef', ''}}}, next_msg())
+    eq({'notification', 'stdout', {0, {''}}}, next_msg())
     eq({'notification', 'exit', {0, 0}}, next_msg())
     os.remove(filename)
 
@@ -168,6 +206,7 @@ describe('jobs', function()
     nvim('command', 'call jobsend(j, "abc\\nxyz")')
     eq({'notification', 'stdout', {0, {'abc', 'xyz'}}}, next_msg())
     nvim('command', "call jobstop(j)")
+    eq({'notification', 'stdout', {0, {''}}}, next_msg())
     eq({'notification', 'exit', {0, 0}}, next_msg())
   end)
 
@@ -186,6 +225,7 @@ describe('jobs', function()
     eq({'notification', 'stdout', {0, {'\n123\n', 'abc\nxyz\n', ''}}},
       next_msg())
     nvim('command', "call jobstop(j)")
+    eq({'notification', 'stdout', {0, {''}}}, next_msg())
     eq({'notification', 'exit', {0, 0}}, next_msg())
   end)
 
@@ -196,19 +236,19 @@ describe('jobs', function()
     eq({'notification', 'stdout', {0, {'some data', 'without\nfinal nl'}}},
       next_msg())
     nvim('command', "call jobstop(j)")
+    eq({'notification', 'stdout', {0, {''}}}, next_msg())
     eq({'notification', 'exit', {0, 0}}, next_msg())
   end)
 
   it('closes the job streams with jobclose', function()
-    if helpers.pending_win32(pending) then return end  -- TODO: Need `cat`.
-    nvim('command', "let j = jobstart(['cat', '-'], g:job_opts)")
+    nvim('command', "let j = jobstart(has('win32') ? ['find', '/v', ''] : ['cat', '-'], g:job_opts)")
     nvim('command', 'call jobclose(j, "stdin")')
-    eq({'notification', 'exit', {0, 0}}, next_msg())
+    eq({'notification', 'stdout', {0, {''}}}, next_msg())
+    eq({'notification', 'exit', {0, iswin() and 1 or 0}}, next_msg())
   end)
 
   it("disallows jobsend on a job that closed stdin", function()
-    if helpers.pending_win32(pending) then return end  -- TODO: Need `cat`.
-    nvim('command', "let j = jobstart(['cat', '-'], g:job_opts)")
+    nvim('command', "let j = jobstart(has('win32') ? ['find', '/v', ''] : ['cat', '-'], g:job_opts)")
     nvim('command', 'call jobclose(j, "stdin")')
     eq(false, pcall(function()
       nvim('command', 'call jobsend(j, ["some data"])')
@@ -221,84 +261,101 @@ describe('jobs', function()
   end)
 
   it('disallows jobstop twice on the same job', function()
-    if helpers.pending_win32(pending) then return end  -- TODO: Need `cat`.
-    nvim('command', "let j = jobstart(['cat', '-'], g:job_opts)")
+    nvim('command', "let j = jobstart(has('win32') ? ['find', '/v', ''] : ['cat', '-'], g:job_opts)")
     neq(0, eval('j'))
     eq(true, pcall(eval, "jobstop(j)"))
     eq(false, pcall(eval, "jobstop(j)"))
   end)
 
   it('will not leak memory if we leave a job running', function()
-    if helpers.pending_win32(pending) then return end  -- TODO: Need `cat`.
     nvim('command', "call jobstart(['cat', '-'], g:job_opts)")
   end)
 
   it('can get the pid value using getpid', function()
-    if helpers.pending_win32(pending) then return end  -- TODO: Need `cat`.
-    nvim('command', "let j =  jobstart(['cat', '-'], g:job_opts)")
+    nvim('command', "let j =  jobstart(has('win32') ? ['find', '/v', ''] : ['cat', '-'], g:job_opts)")
     local pid = eval('jobpid(j)')
-    eq(0,os.execute('ps -p '..pid..' > /dev/null'))
+    neq(NIL, meths.get_proc(pid))
     nvim('command', 'call jobstop(j)')
-    eq({'notification', 'exit', {0, 0}}, next_msg())
-    neq(0,os.execute('ps -p '..pid..' > /dev/null'))
+    eq({'notification', 'stdout', {0, {''}}}, next_msg())
+    if iswin() then
+      expect_msg_seq(
+        -- win64
+        { {'notification', 'exit', {0, 1}}
+        },
+        -- win32
+        { {'notification', 'exit', {0, 15}}
+        }
+      )
+    else
+      eq({'notification', 'exit', {0, 0}}, next_msg())
+    end
+    eq(NIL, meths.get_proc(pid))
   end)
 
   it("do not survive the exit of nvim", function()
-    if helpers.pending_win32(pending) then return end
     -- use sleep, which doesn't die on stdin close
-    nvim('command', "let g:j =  jobstart(['sleep', '1000'], g:job_opts)")
+    nvim('command', "let g:j =  jobstart(has('win32') ? ['ping', '-n', '1001', '127.0.0.1'] : ['sleep', '1000'], g:job_opts)")
     local pid = eval('jobpid(g:j)')
-    eq(0,os.execute('ps -p '..pid..' > /dev/null'))
+    neq(NIL, meths.get_proc(pid))
     clear()
-    neq(0,os.execute('ps -p '..pid..' > /dev/null'))
+    eq(NIL, meths.get_proc(pid))
   end)
 
   it('can survive the exit of nvim with "detach"', function()
-    if helpers.pending_win32(pending) then return end
     nvim('command', 'let g:job_opts.detach = 1')
-    nvim('command', "let g:j = jobstart(['sleep', '1000'], g:job_opts)")
+    nvim('command', "let g:j = jobstart(has('win32') ? ['ping', '-n', '1001', '127.0.0.1'] : ['sleep', '1000'], g:job_opts)")
     local pid = eval('jobpid(g:j)')
-    eq(0,os.execute('ps -p '..pid..' > /dev/null'))
+    neq(NIL, meths.get_proc(pid))
     clear()
-    eq(0,os.execute('ps -p '..pid..' > /dev/null'))
+    neq(NIL, meths.get_proc(pid))
     -- clean up after ourselves
-    os.execute('kill -9 '..pid..' > /dev/null')
+    eq(0, os_kill(pid))
   end)
 
   it('can pass user data to the callback', function()
     nvim('command', 'let g:job_opts.user = {"n": 5, "s": "str", "l": [1]}')
-    nvim('command', [[call jobstart('echo "foo"', g:job_opts)]])
+    nvim('command', [[call jobstart('echo foo', g:job_opts)]])
     local data = {n = 5, s = 'str', l = {1}}
-    eq({'notification', 'stdout', {data, {'foo', ''}}}, next_msg())
+    expect_msg_seq(
+      { {'notification', 'stdout', {data, {'foo', ''}}},
+        {'notification', 'stdout', {data, {''}}},
+      },
+      -- Alternative sequence:
+      { {'notification', 'stdout', {data, {'foo'}}},
+        {'notification', 'stdout', {data, {'', ''}}},
+        {'notification', 'stdout', {data, {''}}},
+      }
+    )
     eq({'notification', 'exit', {data, 0}}, next_msg())
   end)
 
-  it('can omit options', function()
-    if helpers.pending_win32(pending) then return end
-    neq(0, nvim('eval', 'delete(".Xtestjob")'))
-    nvim('command', "call jobstart(['touch', '.Xtestjob'])")
-    nvim('command', "sleep 100m")
-    eq(0, nvim('eval', 'delete(".Xtestjob")'))
-  end)
-
   it('can omit data callbacks', function()
     nvim('command', 'unlet g:job_opts.on_stdout')
-    nvim('command', 'unlet g:job_opts.on_stderr')
     nvim('command', 'let g:job_opts.user = 5')
-    nvim('command', [[call jobstart('echo "foo"', g:job_opts)]])
+    nvim('command', [[call jobstart('echo foo', g:job_opts)]])
     eq({'notification', 'exit', {5, 0}}, next_msg())
   end)
 
   it('can omit exit callback', function()
     nvim('command', 'unlet g:job_opts.on_exit')
     nvim('command', 'let g:job_opts.user = 5')
-    nvim('command', [[call jobstart('echo "foo"', g:job_opts)]])
-    eq({'notification', 'stdout', {5, {'foo', ''}}}, next_msg())
+    nvim('command', [[call jobstart('echo foo', g:job_opts)]])
+    expect_msg_seq(
+      { {'notification', 'stdout', {5, {'foo', ''} } },
+        {'notification', 'stdout', {5, {''} } },
+      },
+      -- Alternative sequence:
+      { {'notification', 'stdout', {5, {'foo'} } },
+        {'notification', 'stdout', {5, {'', ''} } },
+        {'notification', 'stdout', {5, {''} } },
+      }
+    )
   end)
 
   it('will pass return code with the exit event', function()
     nvim('command', 'let g:job_opts.user = 5')
     nvim('command', "call jobstart('exit 55', g:job_opts)")
+    eq({'notification', 'stdout', {5, {''}}}, next_msg())
     eq({'notification', 'exit', {5, 55}}, next_msg())
   end)
 
@@ -314,7 +371,6 @@ describe('jobs', function()
   end)
 
   it('can redefine callbacks being used by a job', function()
-    if helpers.pending_win32(pending) then return end  -- TODO: Need `cat`.
     local screen = Screen.new()
     screen:attach()
     screen:set_default_attr_ids({
@@ -329,7 +385,7 @@ describe('jobs', function()
       \ 'on_stderr': function('g:JobHandler'),
       \ 'on_exit': function('g:JobHandler')
       \ }
-      let job = jobstart('cat -', g:callbacks)
+      let job = jobstart(['cat', '-'], g:callbacks)
     ]])
     wait()
     source([[
@@ -341,6 +397,14 @@ describe('jobs', function()
   end)
 
   it('requires funcrefs for script-local (s:) functions', function()
+    local screen = Screen.new(60, 5)
+    screen:attach()
+    screen:set_default_attr_ids({
+      [1] = {bold = true, foreground = Screen.colors.Blue1},
+      [2] = {foreground = Screen.colors.Grey100, background = Screen.colors.Red},
+      [3] = {bold = true, foreground = Screen.colors.SeaGreen4}
+    })
+
     -- Pass job callback names _without_ `function(...)`.
     source([[
       function! s:OnEvent(id, data, event) dict
@@ -350,33 +414,44 @@ describe('jobs', function()
         \ 'on_stdout': 's:OnEvent',
         \ 'on_stderr': 's:OnEvent',
         \ 'on_exit':   's:OnEvent',
-        \ 'user': 2349
         \ })
     ]])
 
-    -- The behavior is asynchronous, retry until a time limit.
-    helpers.retry(nil, 10000, function()
-      eq("E120:", string.match(eval("v:errmsg"), "E%d*:"))
-    end)
+    screen:expect("{2:E120: Using <SID> not in a script context: s:OnEvent}",nil,nil,nil,true)
   end)
 
   it('does not repeat output with slow output handlers', function()
-    if helpers.pending_win32(pending) then return end
     source([[
       let d = {'data': []}
       function! d.on_stdout(job, data, event) dict
-        call add(self.data, a:data)
+        call add(self.data, Normalize(a:data))
         sleep 200m
       endfunction
       if has('win32')
-        let cmd = '1,2,3,4,5 | foreach-object -process {echo $_; sleep 0.1}'
+        let cmd = 'for /L %I in (1,1,5) do @(echo %I& ping -n 2 127.0.0.1 > nul)'
       else
         let cmd = ['sh', '-c', 'for i in $(seq 1 5); do echo $i; sleep 0.1; done']
       endif
       call jobwait([jobstart(cmd, d)])
-      call rpcnotify(g:channel, 'data', d.data)
     ]])
-    eq({'notification', 'data', {{{'1', ''}, {'2', ''}, {'3', ''}, {'4', ''}, {'5', ''}}}}, next_msg())
+
+    local expected = {'1', '2', '3', '4', '5', ''}
+    local chunks = eval('d.data')
+    local received = {''}
+    for i, chunk in ipairs(chunks) do
+      if i < #chunks then
+        -- if chunks got joined, a spurious [''] callback was not sent
+        neq({''}, chunk)
+      else
+        -- but EOF callback is still sent
+        eq({''}, chunk)
+      end
+      received[#received] = received[#received]..chunk[1]
+      for j = 2, #chunk do
+        received[#received+1] = chunk[j]
+      end
+    end
+    eq(expected, received)
   end)
 
   it('jobstart() works with partial functions', function()
@@ -388,9 +463,16 @@ describe('jobs', function()
     endfunction
     let Callback = function('PrintArgs', ["foo", "bar"])
     let g:job_opts = {'on_stdout': Callback}
-    call jobstart('echo "some text"', g:job_opts)
+    call jobstart('echo some text', g:job_opts)
     ]])
-    eq({'notification', '1', {'foo', 'bar', {'some text', ''}, 'stdout'}}, next_msg())
+    expect_msg_seq(
+      { {'notification', '1', {'foo', 'bar', {'some text', ''}, 'stdout'}},
+      },
+      -- Alternative sequence:
+      { {'notification', '1', {'foo', 'bar', {'some text'}, 'stdout'}},
+        {'notification', '1', {'foo', 'bar', {'', ''}, 'stdout'}},
+      }
+    )
   end)
 
   it('jobstart() works with closures', function()
@@ -401,23 +483,48 @@ describe('jobs', function()
           return {id, data, event -> rpcnotify(g:channel, '1', a1, a2, Normalize(data), event)}
       endfun
       let g:job_opts = {'on_stdout': MkFun()}
-      call jobstart('echo "some text"', g:job_opts)
+      call jobstart('echo some text', g:job_opts)
     ]])
-    eq({'notification', '1', {'foo', 'bar', {'some text', ''}, 'stdout'}}, next_msg())
+    expect_msg_seq(
+      { {'notification', '1', {'foo', 'bar', {'some text', ''}, 'stdout'}},
+      },
+      -- Alternative sequence:
+      { {'notification', '1', {'foo', 'bar', {'some text'}, 'stdout'}},
+        {'notification', '1', {'foo', 'bar', {'', ''}, 'stdout'}},
+      }
+    )
   end)
 
   it('jobstart() works when closure passed directly to `jobstart`', function()
     source([[
       let g:job_opts = {'on_stdout': {id, data, event -> rpcnotify(g:channel, '1', 'foo', 'bar', Normalize(data), event)}}
-      call jobstart('echo "some text"', g:job_opts)
+      call jobstart('echo some text', g:job_opts)
     ]])
-    eq({'notification', '1', {'foo', 'bar', {'some text', ''}, 'stdout'}}, next_msg())
+    expect_msg_seq(
+      { {'notification', '1', {'foo', 'bar', {'some text', ''}, 'stdout'}},
+      },
+      -- Alternative sequence:
+      { {'notification', '1', {'foo', 'bar', {'some text'}, 'stdout'}},
+        {'notification', '1', {'foo', 'bar', {'', ''}, 'stdout'}},
+      }
+    )
   end)
 
   describe('jobwait', function()
+    before_each(function()
+      if iswin() then
+        helpers.set_shell_powershell()
+      end
+    end)
+
     it('returns a list of status codes', function()
       source([[
-      call rpcnotify(g:channel, 'wait', jobwait([
+      call rpcnotify(g:channel, 'wait', jobwait(has('win32') ? [
+      \  jobstart('Start-Sleep -Milliseconds 100; exit 4'),
+      \  jobstart('Start-Sleep -Milliseconds 300; exit 5'),
+      \  jobstart('Start-Sleep -Milliseconds 500; exit 6'),
+      \  jobstart('Start-Sleep -Milliseconds 700; exit 7')
+      \  ] : [
       \  jobstart('sleep 0.10; exit 4'),
       \  jobstart('sleep 0.110; exit 5'),
       \  jobstart('sleep 0.210; exit 6'),
@@ -437,7 +544,12 @@ describe('jobs', function()
         endif
         let g:exits += 1
       endfunction
-      call jobwait([
+      call jobwait(has('win32') ? [
+      \  jobstart('Start-Sleep -Milliseconds 100; exit 5', g:dict),
+      \  jobstart('Start-Sleep -Milliseconds 300; exit 5', g:dict),
+      \  jobstart('Start-Sleep -Milliseconds 500; exit 5', g:dict),
+      \  jobstart('Start-Sleep -Milliseconds 700; exit 5', g:dict)
+      \  ] : [
       \  jobstart('sleep 0.010; exit 5', g:dict),
       \  jobstart('sleep 0.030; exit 5', g:dict),
       \  jobstart('sleep 0.050; exit 5', g:dict),
@@ -450,7 +562,12 @@ describe('jobs', function()
 
     it('will return status codes in the order of passed ids', function()
       source([[
-      call rpcnotify(g:channel, 'wait', jobwait([
+      call rpcnotify(g:channel, 'wait', jobwait(has('win32') ? [
+      \  jobstart('Start-Sleep -Milliseconds 700; exit 4'),
+      \  jobstart('Start-Sleep -Milliseconds 500; exit 5'),
+      \  jobstart('Start-Sleep -Milliseconds 300; exit 6'),
+      \  jobstart('Start-Sleep -Milliseconds 100; exit 7')
+      \  ] : [
       \  jobstart('sleep 0.070; exit 4'),
       \  jobstart('sleep 0.050; exit 5'),
       \  jobstart('sleep 0.030; exit 6'),
@@ -464,16 +581,29 @@ describe('jobs', function()
       source([[
       call rpcnotify(g:channel, 'wait', jobwait([
       \  -10,
-      \  jobstart('sleep 0.01; exit 5'),
+      \  jobstart((has('win32') ? 'Start-Sleep -Milliseconds 100' : 'sleep 0.01').'; exit 5'),
       \  ]))
       ]])
       eq({'notification', 'wait', {{-3, 5}}}, next_msg())
     end)
 
-    it('will return -2 when interrupted', function()
+    it('will return -2 when interrupted without timeout', function()
       feed_command('call rpcnotify(g:channel, "ready") | '..
               'call rpcnotify(g:channel, "wait", '..
-              'jobwait([jobstart("sleep 10; exit 55")]))')
+              'jobwait([jobstart("'..
+              (iswin() and 'Start-Sleep 10' or 'sleep 10')..
+              '; exit 55")]))')
+      eq({'notification', 'ready', {}}, next_msg())
+      feed('<c-c>')
+      eq({'notification', 'wait', {{-2}}}, next_msg())
+    end)
+
+    it('will return -2 when interrupted with timeout', function()
+      feed_command('call rpcnotify(g:channel, "ready") | '..
+              'call rpcnotify(g:channel, "wait", '..
+              'jobwait([jobstart("'..
+              (iswin() and 'Start-Sleep 10' or 'sleep 10')..
+              '; exit 55")], 10000))')
       eq({'notification', 'ready', {}}, next_msg())
       feed('<c-c>')
       eq({'notification', 'wait', {{-2}}}, next_msg())
@@ -497,7 +627,8 @@ describe('jobs', function()
         elseif self.state == 2
           let self.state = 3
           call jobsend(a:id, "line3\n")
-        else
+        elseif self.state == 3
+          let self.state = 4
           call rpcnotify(g:channel, 'w', printf('job %d closed', self.counter))
           call jobclose(a:id, 'stdin')
         endif
@@ -527,20 +658,22 @@ describe('jobs', function()
     end)
 
     describe('with timeout argument', function()
-      if helpers.pending_win32(pending) then return end
       it('will return -1 if the wait timed out', function()
         source([[
         call rpcnotify(g:channel, 'wait', jobwait([
         \  jobstart('exit 4'),
-        \  jobstart('sleep 10; exit 5'),
-        \  ], 100))
+        \  jobstart((has('win32') ? 'Start-Sleep 10' : 'sleep 10').'; exit 5'),
+        \  ], has('win32') ? 3000 : 100))
         ]])
         eq({'notification', 'wait', {{4, -1}}}, next_msg())
       end)
 
       it('can pass 0 to check if a job exists', function()
         source([[
-        call rpcnotify(g:channel, 'wait', jobwait([
+        call rpcnotify(g:channel, 'wait', jobwait(has('win32') ? [
+        \  jobstart('Start-Sleep -Milliseconds 50; exit 4'),
+        \  jobstart('Start-Sleep -Milliseconds 300; exit 5'),
+        \  ] : [
         \  jobstart('sleep 0.05; exit 4'),
         \  jobstart('sleep 0.3; exit 5'),
         \  ], 0))
@@ -552,6 +685,7 @@ describe('jobs', function()
 
   -- FIXME need to wait until jobsend succeeds before calling jobstop
   pending('will only emit the "exit" event after "stdout" and "stderr"', function()
+    nvim('command', "let g:job_opts.on_stderr  = function('s:OnEvent')")
     nvim('command', "let j = jobstart(['cat', '-'], g:job_opts)")
     local jobid = nvim('eval', 'j')
     nvim('eval', 'jobsend(j, "abcdef")')
@@ -561,13 +695,60 @@ describe('jobs', function()
   end)
 
   it('cannot have both rpc and pty options', function()
-    if helpers.pending_win32(pending) then return end  -- TODO: Need `cat`.
     command("let g:job_opts.pty = v:true")
     command("let g:job_opts.rpc = v:true")
-    local _, err = pcall(command, "let j = jobstart(['cat', '-'], g:job_opts)")
+    local _, err = pcall(command, "let j = jobstart(has('win32') ? ['find', '/v', ''] : ['cat', '-'], g:job_opts)")
     ok(string.find(err, "E475: Invalid argument: job cannot have both 'pty' and 'rpc' options set") ~= nil)
   end)
 
+  it('does not crash when repeatedly failing to start shell', function()
+    source([[
+      set shell=nosuchshell
+      func! DoIt()
+        call jobstart('true')
+        call jobstart('true')
+      endfunc
+    ]])
+    -- The crash only triggered if both jobs are cleaned up on the same event
+    -- loop tick. This is also prevented by try-block, so feed must be used.
+    feed_command("call DoIt()")
+    feed('<cr>') -- press RETURN
+    eq(2,eval('1+1'))
+  end)
+
+  it('jobstop() kills entire process tree #6530', function()
+    -- XXX: Using `nvim` isn't a good test, it reaps its children on exit.
+    -- local c = 'call jobstart([v:progpath, "-u", "NONE", "-i", "NONE", "--headless"])'
+    -- local j = eval("jobstart([v:progpath, '-u', 'NONE', '-i', 'NONE', '--headless', '-c', '"
+    --                ..c.."', '-c', '"..c.."'])")
+
+    -- Create child with several descendants.
+    local sleep_cmd = (iswin()
+      and 'ping -n 31 127.0.0.1'
+      or  'sleep 30')
+    local j = eval("jobstart('"..sleep_cmd..' | '..sleep_cmd..' | '..sleep_cmd.."')")
+    local ppid = funcs.jobpid(j)
+    local children
+    retry(nil, nil, function()
+      children = meths.get_proc_children(ppid)
+      eq(3, #children)
+    end)
+    -- Assert that nvim_get_proc() sees the children.
+    for _, child_pid in ipairs(children) do
+      local info = meths.get_proc(child_pid)
+      -- eq((iswin() and 'nvim.exe' or 'nvim'), info.name)
+      eq(ppid, info.ppid)
+    end
+    -- Kill the root of the tree.
+    funcs.jobstop(j)
+    -- Assert that the children were killed.
+    retry(nil, nil, function()
+      for _, child_pid in ipairs(children) do
+        eq(NIL, meths.get_proc(child_pid))
+      end
+    end)
+  end)
+
   describe('running tty-test program', function()
     if helpers.pending_win32(pending) then return end
     local function next_chunk()
@@ -638,7 +819,7 @@ describe('jobs', function()
       -- there won't be any more messages, and the test would hang.
       helpers.sleep(100)
       local err = exc_exec('call jobpid(j)')
-      eq('Vim(call):E900: Invalid job id', err)
+      eq('Vim(call):E900: Invalid channel id', err)
 
       -- cleanup
       eq(other_pid, eval('jobpid(' .. other_jobid .. ')'))
