@@ -16,8 +16,8 @@ describe('sign', function()
         nvim('command', 'sign place 34 line=3 name=Foo buffer='..buf2)
         -- now unplace without specifying a buffer
         nvim('command', 'sign unplace 34')
-        eq("\n--- Signs ---\n", nvim('command_output', 'sign place buffer='..buf1))
-        eq("\n--- Signs ---\n", nvim('command_output', 'sign place buffer='..buf2))
+        eq("--- Signs ---\n", nvim('command_output', 'sign place buffer='..buf1))
+        eq("--- Signs ---\n", nvim('command_output', 'sign place buffer='..buf2))
       end)
     end)
   end)
